#################################################################################################
##### - All sequences need to live in a folder named after the resolution of the images     #####
##### - Example - D:/Perforce/EXR_Sequences/Smoke/2048x2048/                                #####
#####             D:/Perforce/EXR_Sequences/Smoke/1024x1024/                                #####
#####             D:/Perforce/EXR_Sequences/Smoke/512x512/                                  #####
##### - For GPU optimized streaming into Unreal all Exr Sequences should be uncompressed    #####
##### - A mix of uncompressed and uncompressed in the mip levels will break the chain       #####
##### - Just select your exr seqeuence, set the number of mip levels desired                #####
#####   at the top of the script, select the read nodes you want to mip, and execute        #####
##### - All the necessary reformats and write nodes will be created for you in nuke         #####
##### - All the necessary resolution folders for each mip render will be created for you    #####
##### - The paths of the mip renders will be based on the selected read node on execution   #####
#################################################################################################

import nuke
import os

##########################################
#### How many mip levels do you want? ####
##########################################
mipLevels = 3
##########################################
##########################################

##### Grabs node selection ####
selectedRead = nuke.selectedNodes()
addLevel = mipLevels + 1

#### Gets Height and Width of Image Sequence ####
def getHeightWidth(read):
    getFormat = []
    getHeight = []
    getWidth = []
    getFormat = read.format()
    getHeight = getFormat.height()
    getWidth = getFormat.width()
    dirResName = str(getWidth) + 'x' + str(getHeight)
    return dirResName

def getFilePathName(readNode):
    getName = readNode['file'].value()
    return getName

#### Create directory ####
def createDirectories(readNode,read):
    getNameLocal = getFilePathName(readNode)
    getHeightWidthLocal = getHeightWidth(read)
    getSequenceName = []
    parentPath = []
    dirResName = []
    dirName = []
    setRenderPathName = []
    getSequenceName = getNameLocal.split('/')[-1]
    parentPath = getNameLocal.split(getSequenceName)[0]
    dirName = parentPath + getHeightWidthLocal

#### Sets Render Path Name #####
    setRenderPathName = dirName + '/' + getSequenceName 
    isThere = os.path.isdir(dirName)
    if isThere == False:
        os.makedirs(dirName)
    return setRenderPathName

#### Creates reformat ####
def createReformatNodes(connectReformat):
    createScale = nuke.nodes.Reformat()
    createScale['type'].setValue("scale")
    createScale['scale'].setValue(0.5)
    createScale.connectInput(1,connectReformat)
    return createScale

#### Creates write node ####
def createWriteNodes(path,connect):   
    createWrite = nuke.nodes.Write()
    createWrite['file'].setValue(path)
    createWrite['file_type'].setValue('exr')
    createWrite['compression'].setValue('none')
    createWrite.connectInput(1,connect)
    return createWrite

#### Generate Tree ####
if len(selectedRead) > 0:
    for x in selectedRead:
        getFilePathName(x)
        for index in range(addLevel):
            if index == 0:
                getHeightWidth(x)
                setPathLocal = createDirectories(x,x)
                createWriteLocal = createWriteNodes(setPathLocal,x)
            else:
                createScaleLocal = createReformatNodes(createWriteLocal)
                getHeightWidth(createScaleLocal)
                setPathLocal = createDirectories(x,createScaleLocal)
                createWriteLocal = createWriteNodes(setPathLocal,createScaleLocal)
else:
    nuke.alert("Nothing selected. Please selected your EXR sequence READ NODES to generate mipmaps")